from torch import nn


class SimSiamLoss(nn.Module):
    def __init__(self, version='simplified'):
        super().__init__()
        self.ver = version

    def asymmetric_loss(self, p, z):
        if self.ver == 'original':
            z = z.detach()  # stop gradient

            p = nn.functional.normalize(p, dim=1)
            z = nn.functional.normalize(z, dim=1)

            return -(p * z).sum(dim=1).mean()

        elif self.ver == 'simplified':
            z = z.detach()  # stop gradient
            return - nn.functional.cosine_similarity(p, z, dim=-1).mean()

    def forward(self, outputs):
        loss1 = self.asymmetric_loss(outputs['p1'], outputs['z2'])
        loss2 = self.asymmetric_loss(outputs['p2'], outputs['z1'])

        return 0.5 * loss1 + 0.5 * loss2
